@@ -31,6 +31,7 @@ import org.apache.flink.cdc.common.event.DataChangeEvent;
 import org.apache.flink.cdc.common.event.DropColumnEvent;
 import org.apache.flink.cdc.common.event.Event;
 import org.apache.flink.cdc.common.event.TableId;
+import org.apache.flink.cdc.common.exceptions.SchemaEvolveException;
 import org.apache.flink.cdc.common.schema.Column;
 import org.apache.flink.cdc.common.schema.Schema;
 import org.apache.flink.cdc.common.types.DataTypes;
@@ -129,7 +130,7 @@ public class PaimonSinkITCase {
                 .dropDatabase(TEST_DATABASE, true, true);
     }
 
-    private List<Event> createTestEvents() {
+    private List<Event> createTestEvents() throws SchemaEvolveException {
         List<Event> testEvents = new ArrayList<>();
         // create table
         Schema schema =
@@ -137,6 +138,7 @@ public class PaimonSinkITCase {
                         .physicalColumn("col1", DataTypes.STRING())
                         .physicalColumn("col2", DataTypes.STRING())
                         .primaryKey("col1")
+                        .option("bucket", "1")
                         .build();
         CreateTableEvent createTableEvent = new CreateTableEvent(table1, schema);
         testEvents.add(createTableEvent);
@@ -171,7 +173,7 @@ public class PaimonSinkITCase {
     @ValueSource(strings = {"filesystem", "hive"})
     public void testSinkWithDataChange(String metastore)
             throws IOException, InterruptedException, Catalog.DatabaseNotEmptyException,
-                    Catalog.DatabaseNotExistException {
+                    Catalog.DatabaseNotExistException, SchemaEvolveException {
         initialize(metastore);
         PaimonSink<Event> paimonSink =
                 new PaimonSink<>(
@@ -257,7 +259,7 @@ public class PaimonSinkITCase {
     @ValueSource(strings = {"filesystem", "hive"})
     public void testSinkWithSchemaChange(String metastore)
             throws IOException, InterruptedException, Catalog.DatabaseNotEmptyException,
-                    Catalog.DatabaseNotExistException {
+                    Catalog.DatabaseNotExistException, SchemaEvolveException {
         initialize(metastore);
         PaimonSink<Event> paimonSink =
                 new PaimonSink(
@@ -392,7 +394,7 @@ public class PaimonSinkITCase {
     @ValueSource(strings = {"filesystem", "hive"})
     public void testSinkWithMultiTables(String metastore)
             throws IOException, InterruptedException, Catalog.DatabaseNotEmptyException,
-                    Catalog.DatabaseNotExistException {
+                    Catalog.DatabaseNotExistException, SchemaEvolveException {
         initialize(metastore);
         PaimonSink<Event> paimonSink =
                 new PaimonSink<>(
@@ -407,6 +409,7 @@ public class PaimonSinkITCase {
                         .physicalColumn("col1", DataTypes.STRING())
                         .physicalColumn("col2", DataTypes.STRING())
                         .primaryKey("col1")
+                        .option("bucket", "1")
                         .build();
         CreateTableEvent createTableEvent = new CreateTableEvent(table2, schema);
         testEvents.add(createTableEvent);
